% Script for processing water elevation data from the EMS logger
% Script developed by H.J. Verhagen, Delft University of Technology
% April 2013
clear;
clc;
%==========================================================================
%define basic variables and other input
g=9.81;
Rho=1030;        % Density of water
TanAlfa = 0.001; % bedslope
AverageDepth=5;  % Average waterdepth
interval=.10;    % interval is the sample frequency interval of the sensor
                 % do not change this value unless another wavelogger is used
                 % .25
% additional input data for spectrum
%===================================
M = 50;          % higher values of M give more smoothing of the spectrum
                 % low value for M is 20, high value for M is 100
high = 0.4;      % highest frequency for plots  default high=0.4
low = 0.05;      % lowest frequency for plots   default low =0.05
%==========================================================================
     [FileName,PathName] = uigetfile('*.txt','Select the wavelogger file');
     fid=fopen([PathName,FileName]);
     [pathstr, heading,ext]=fileparts(FileName);

     % heading is caption of figures, default heading=FileName
     % if needed assign other value to heading here
     % heading = 'fill in your heading';

DATA=textscan(fid,'%f');

fclose(fid);
eta=DATA{1};
n=length(eta);                      

tottime=n*interval;   %calculate total duration of observation in sec
ttime=(interval:interval:tottime); %create array with time
time=ttime.';          %change orientation of matrix
% calculate regression coefficients to compensate for change in waterlevel
% during the observations

%============================================================================
%Time domain analysis
period=0;
start=1;
  %loop to find individual waves from waterlevel record (eta)
  counter2=0;
  etamax=0;
  etamin=0;
  for k=1:n-1
    if(eta(k+1)*eta(k)<0 && eta(k+1)<0)  %new wave starts at k+1
        counter2=counter2+1;
        H(counter2)=etamax-etamin;
        TT(counter2)=period;
        period=0;
        etamax=0;
        etamin=0;
    end;
    if(eta(k)>etamax)  etamax=eta(k);
      end;
    if (eta(k)<etamin) etamin=eta(k);
      end;
    period=period+interval;  
  end;
  % vector H contains all individual waves
  % sort H to make exceedance graph

 figure;
 subplot(4,1,1);
 plot(time,eta);
 xlabel('time (sec)');
 ylabel('waterlevel (m)');
 title([heading]);

counter=length(H);
for k=1 : counter    
prob(k)= (counter-k+1)/counter;
end;

Hsorted=sort(H);
Hmean=mean(Hsorted);
Hrms = sqrt(mean(Hsorted.^2)); % rms wave height

from=round(2*counter/3);
to=round(counter);
Hs=mean(Hsorted(from:to)); %H 1/3, or significant wave height

Tsorted=sort(TT);
Tmean=mean(TT);
from=round(2*counter/3);
to=round(counter);
T3=mean(Tsorted(from:to)); %T 1/3, or significant wave peirodt

%calculation of the H2%
Twopercent=round(counter-counter/50);
if (Twopercent==counter)
  H2percentMeasured=NaN;
  else
  H2percentMeasured=Hsorted(Twopercent);
  end;
H2percRayleigh=1.4*Hs;
%caculation of the H1%
Onepercent=round(counter-counter/100);
if (Onepercent==counter)
  H1percentMeasured=NaN;
  else
  H1percentMeasured=Hsorted(Onepercent);
  end;
H1percRayleigh=1.5*Hs;
%calculation of the H0.1%
Onepermille=round(counter-counter/1000);
if (Onepermille==counter)
  H1permilleMeasured=NaN;
  else
  H1permilleMeasured=Hsorted(Onepermille);
  end;
H1permilleRayleigh=1.85*Hs;

% Compositie weibull ditribution for shallow water
% source: Wave height distributions on shallow foreshores 
%         Coastal Engineering, Volume 40, Issue 3, June 2000, Pages 161-182
%         Jurjen A Battjes, Heiko W Groenendijk

Htr=(0.35+5.8*TanAlfa)*AverageDepth; % Transistion wave height acc. to BG (eq. 8)

H3=Hrms*BG(Htr,3);
H10=Hrms*BG(Htr,10);
H2=Hrms*BG(Htr,0.02);
H1=Hrms*BG(Htr,0.01);
H01=Hrms*BG(Htr,0.001);

%plot Rayleigh graph

 h = axes('Position',[0 0 1 1],'Visible','off');
 axes('Position',[.45 .40 .45 .30])

  for i=1:counter
     probsorted2(i)=1-i/(counter+1);
  end;
 maxh=1.3*Hsorted(counter);
 step=1;
 if (maxh<5.1) step=0.5;end;
 if (maxh<2.6) step=0.25;end;
 xmark=[0:step:maxh];
 coef=Rayleigh(Hsorted,probsorted2,xmark,Hrms,Htr);
 title('Wave height distribution (m) ');
 string=num2str(Hmean,2);
 str(1)= {['H_{mean} (m)             ',string]};
 string=num2str(Hrms,2);
 str(2)= {['H_{rms} (m)              ',string]};
 string=num2str(Hs,2);
 str(3)= {['H_{s} (m)                 ',string]};
 string=num2str(Tmean,2);
 str(4)= {['T_{mean} (s)             ',string]};
 string=num2str(T3,2);
 str(5)= {['T_{1/3} (s)               ',string]};
 string=num2str(Htr,2);
 str(6)= {['H_{tr}(m)(acc BG)  '   ,string]};
 string=num2str(1/TanAlfa);
 str(7)= {['Bed slope 1:',string]};
 str(8)={'              obs    Rayleigh  BG'};
 string=num2str(H2percentMeasured,3); stringB=num2str(H2percRayleigh,3);stringC=num2str(H2,3);
 str(9)= {['H_{2%}(m)   ',string,'    ',stringB,'      ',stringC]};
 string=num2str(H1percentMeasured,3); stringB=num2str(H1percRayleigh,3);;stringC=num2str(H1,3);
 str(10)= {['H_{1%}(m)   ',string,'    ',stringB,'      ',stringC]};
 string=num2str(H1permilleMeasured,3); stringB=num2str(H1permilleRayleigh,3);;stringC=num2str(H01,3);
 str(11)= {['H_{0.1%}(m) ',string,'    ',stringB,'      ',stringC]};
 set(gcf,'CurrentAxes',h)
 text(.1,.55,str,'FontSize',8)
 
%===================================================================
% simple script utilizing crosgk (by G. Klopman) to obtain
% spectral estimate
%===================================================================
% data  contains the data 
% N  is the number of samples per data segment (power of 2)
% M  is the number of frequency bins over which is smoothed (optional),
%    no smoothing for M=1 (default)
% DT is the time step (optional), default DT=1
% DW is the data window type (optional): DW = 1 for Hann window (default)
%                                        DW = 2 for rectangular window
% stats : display resolution, degrees of freedom (optimal, YES=1, NO=0)
%
% Output:
% P contains the (cross-)spectral estimates: column 1 = Pxx, 2 = Pyy, 3 = Pxy
% F contains the frequencies at which P is given load time series
             
DT = interval;
    data = eta;
    [P,F,dof]=crosgk(data,data,length(data),M,DT,1,0);

  m0=0;   %zero-th moment
  m1=0;   %first moment
  m2=0;   %second moment
  m01=0;  %first negative moment {m(-1,0)}
  deltaF= F(31)-F(30);
  
  emax=0;
  prmax=0;  %maximum value of pressure energy
  % claculation loop to transform pressure spectrum to energy spectrum and
  % to calculate the moments of the spectrum
  % low and high frequencies are deleted, range from 200 to 0.2*max
  % frequency bin, 200 means f= 200*deltaF, which is approx. 30 seconds
  % 0.2*length(F)*deltaF = 0.4, so Tmin - 2.5 seconds
  
    energy=P;   %energy density in Hz/m2
  for i=1:2000    %50:2000
      T=1/F(i);
      pr=sqrt(P(i,1));   %pr=pressure value in pressure spectrum
      L0=1.56*T*T;
      if (AverageDepth/L0<0.36) 
          L=sqrt(g*AverageDepth)*(1-AverageDepth/L0)*T;
          else
          L=L0;
          end;
      if energy(i)>emax 
          emax=energy(i);
          Tpeak=T;
          end;
      m0 =m0 +energy(i)*deltaF;
      m1 =m1 +energy(i)*deltaF*F(i);
      m2 =m2 +energy(i)*deltaF*F(i)^2;
      if (F(i)>0)
          m01=m01+energy(i)*deltaF/F(i);
          end;
  end;
  
  Hm0=4*sqrt(m0);    %one may assume Hs = Hm0
  Hrmss=sqrt(8*m0);   %root mean square height from spectrum
  Tm=sqrt(m0/m2);    %spectral approximation of mean period
  T01=m0/m1;         %Period based on first moment 
  T10=m01/m0;        %Period based on first negative moment
  
  % plot spectrum and print output
   h = axes('Position',[0 0 1 1],'Visible','off');
   axes('Position',[.45 .05 .45 .20])
   plot(F,energy)
   axis([low high 0 10000]) 
   axis 'auto y'
   xlabel('frequency [Hz]');
   ylabel ('Energy m^2/Hz');
   title('energy spectrum ');
   string=num2str(Hrmss,2);
   str(1)= {['H_{rms} (m) ',string]};
   string=num2str(Hm0,2);
   str(2)= {['H_{m0} (m)  ',string]};
   string=num2str(Tm,2);
   str(3)= {['T_{m} (s)    ',string]};
   string=num2str(T01,2);
   str(4)= {['T_{m0,1} (s) ',string]};
   string=num2str(T10,2);
   str(5)= {['T_{m-1,0}(s) ',string]};
   string=num2str(Tpeak,2);
   str(6)= {['T_{peak}(s)  ',string]};
   str(7)={' '};
   str(8)={' '};
   string=num2str(AverageDepth,2);
   str(9)={['Average depth (m) ',string]};
   string=num2str(Rho);
   str(10)={['water density (kgm^{3})    ',string]};
   str(11)={' '};
   set(gcf,'CurrentAxes',h)
   text(.1,.15,str,'FontSize',8)




